@@ -17,8 +17,8 @@ class DHCPDAddressConflict(EventClass):
     priority = "WARNING"
     subject_template="DHCPD detects address conflict: {{ip}}"
     body_template="""DHCPD detects address conflict: {{ip}}"""
-    repeat_suppression=False
-    repeat_suppression_interval=3600
+    repeat_suppression=True
+    repeat_suppression_interval=600
     trigger=None
     class Vars:
         ip=Var(required=True,repeat=False)
