@@ -12,11 +12,12 @@ from django.shortcuts import get_object_or_404
 from django.http import HttpResponse,HttpResponseRedirect,HttpResponseForbidden
 from django import forms
 
-from noc.ip.models import VRFGroup,VRF,IPv4BlockAccess,IPv4Block,IPv4Address
+from noc.ip.models import VRFGroup,VRF,IPv4BlockAccess,IPv4Block,IPv4Address,IPv4AddressRange
 from noc.peer.models import AS
 from noc.lib.render import render,render_success,render_failure
 from noc.lib.validators import is_rd,is_cidr,is_int,is_ipv4,is_fqdn
-from noc.lib.ip import normalize_prefix,contains
+from noc.lib.ip import normalize_prefix,contains,in_range,free_blocks
+from noc.lib.colors import get_colors
 from noc.settings import config
 import csv,cStringIO,datetime,subprocess,re
 
@@ -40,8 +41,46 @@ def vrf_index(request,vrf_id,prefix="0.0.0.0/0"):
     prefixes=vrf.prefixes(prefix)
     can_allocate=IPv4BlockAccess.check_write_access(request.user,vrf,prefix)
     prefix=vrf.prefix(prefix)
+    has_children=prefix.has_children
+    total=prefix.size
+    block_info=[
+        ("Net",         prefix.prefix),
+        ("Maintainers", ", ".join([str(u) for u in prefix.maintainers])),
+        ("Netmask",     prefix.netmask),
+        ("Broadcast",   prefix.broadcast),
+        ("Wildcard",    prefix.wildcard),
+        ("Size",        total),
+    ]
+    ranges=None
+    all_addresses=None
+    if not has_children:
+        used=prefix.address_count
+        free=total-used
+        block_info+=[
+            ("Addresses Used",used),
+            ("Addresses Free",free),
+        ]
+        ranges=list(prefix.ranges)
+        colors=get_colors(len(ranges))
+        rc=[]
+        for r,c in zip(ranges,colors):
+            r.color=c
+            rc+=[(r.from_ip,r.to_ip,c)]
+        all_addresses=[]
+        for a in prefix.all_addresses:
+            try:
+                ip=a.ip
+            except:
+                ip=a
+            c=None
+            for from_ip,to_ip,cl in rc:
+                if in_range(ip,from_ip,to_ip):
+                    c=cl
+                    break
+            all_addresses+=[(c,a)]
     return render(request,"ip/vrf_index.html",{"vrf":vrf,"parents":parents,"prefixes":prefixes,"prefix":prefix,
-                        "can_allocate":can_allocate})
+                        "can_allocate":can_allocate,"block_info":block_info,"has_children":has_children,
+                        "ranges":ranges,"all_addresses":all_addresses})
 ##
 ## Allocate new block handler
 ##
@@ -50,13 +89,33 @@ class AllocateBlockForm(forms.Form):
     description=forms.CharField(label="description",required=True)
     asn=forms.ModelChoiceField(label="ASN",queryset=AS.objects.all(),required=True)
     tt=forms.IntegerField(label="TT #",required=False)
+    def __init__(self,data=None,initial=None,vrf=None,block_id=None):
+        super(AllocateBlockForm,self).__init__(data=data,initial=initial)
+        self.vrf=vrf
+        self.block_id=block_id
     def clean_prefix(self):
         if not is_cidr(self.cleaned_data["prefix"]):
             raise forms.ValidationError("Invalid prefix")
-        return normalize_prefix(self.cleaned_data["prefix"])
+        prefix=normalize_prefix(self.cleaned_data["prefix"])
+        # Check for duplocation
+        q=IPv4Block.objects.filter(prefix=prefix)
+        if self.vrf.vrf_group.unique_addresses:
+            q=q.filter(vrf__in=self.vrf.vrf_group.vrf_set.all())
+        else:
+            q=q.filter(vrf=self.vrf)
+        if self.block_id:
+            q=q.exclude(id=self.block_id)
+        if q.count()>0:
+            raise forms.ValidationError("Block is already present")
+        return prefix
+
+rx_referer_prefix=re.compile(r"^.+/(?P<prefix>\d+\.\d+\.\d+\.\d+/\d+)/$")
 
 def allocate_block(request,vrf_id,prefix=None):
     vrf=get_object_or_404(VRF,id=int(vrf_id))
+    suggestions=[]
+    parent=prefix
+    block_id=None
     if prefix:
         assert is_cidr(prefix)
         block=get_object_or_404(IPv4Block,vrf=vrf,prefix=prefix)
@@ -67,11 +126,23 @@ def allocate_block(request,vrf_id,prefix=None):
             "tt"          : block.tt
         }
         p="/"+prefix
+        block_id=block.id
     else:
+        match=rx_referer_prefix.match(request.META.get("HTTP_REFERER",""))
+        if match and is_cidr(match.group(1)):
+            # Suggest blocks
+            parent=match.group(1)
+            free=[(x.split("/")[0],int(x.split("/")[1])) for x in free_blocks(parent,[p.prefix for p in vrf.prefixes(top=parent)])]
+            free=sorted(free,lambda x,y: -cmp(x[1],y[1]))
+            for m in range(int(parent.split("/")[1])+1,31):
+                for fp,fm in free:
+                    if fm<=m:
+                        suggestions+=["%s/%d"%(fp,m)]
+                        break
         initial={}
         p=""
     if request.POST:
-        form=AllocateBlockForm(request.POST)
+        form=AllocateBlockForm(request.POST,vrf=vrf,block_id=block_id)
         if form.is_valid():
             if not IPv4BlockAccess.check_write_access(request.user,vrf,form.cleaned_data["prefix"]):
                 return HttpResponseForbidden("Permission denied")
@@ -90,7 +161,7 @@ def allocate_block(request,vrf_id,prefix=None):
             return HttpResponseRedirect("/ip/%d/%s/"%(vrf.id,block.prefix))
     else:
         form=AllocateBlockForm(initial=initial)
-    return render(request,"ip/allocate_block.html",{"vrf":vrf,"form":form,"p":p})
+    return render(request,"ip/allocate_block.html",{"vrf":vrf,"form":form,"p":p,"suggestions":suggestions,"parent":parent})
 ##
 ## Deallocate block handler
 ##
@@ -113,6 +184,10 @@ class AssignAddressForm(forms.Form):
     ip=forms.CharField(label="IP",required=True)
     description=forms.CharField(label="Description",required=False)
     tt=forms.IntegerField(label="TT #",required=False)
+    def __init__(self,data=None,initial=None,vrf=None,address_id=None):
+        super(AssignAddressForm,self).__init__(data=data,initial=initial)
+        self.vrf=vrf
+        self.address_id=address_id
     def clean_fqdn(self):
         if not is_fqdn(self.cleaned_data["fqdn"]):
             raise forms.ValidationError("Invalid FQDN")
@@ -120,16 +195,31 @@ class AssignAddressForm(forms.Form):
     def clean_ip(self):
         if not is_ipv4(self.cleaned_data["ip"]):
             raise forms.ValidationError("Invalid IP Address")
-        return self.cleaned_data["ip"]
+        if IPv4AddressRange.is_range_locked(self.vrf,self.cleaned_data["ip"]):
+            raise forms.ValidationError("IP Address Range is locked")
+        # Check for duplications
+        ip=self.cleaned_data["ip"]
+        q=IPv4Address.objects.filter(ip=ip)
+        if self.vrf.vrf_group.unique_addresses:
+            q=q.filter(vrf__in=self.vrf.vrf_group.vrf_set.all())
+        else:
+            q=q.filter(vrf=self.vrf)
+        if self.address_id:
+            q=q.exclude(id=self.address_id)
+        if q.count()>0:
+            raise forms.ValidationError("IPv4 Address is already present")
+        return ip
 
 rx_url_cidr=re.compile(r"^.*/(\d+\.\d+\.\d+\.\d+/\d+)/$")
 
 def assign_address(request,vrf_id,ip=None,new_ip=None):
     vrf=get_object_or_404(VRF,id=int(vrf_id))
     parents=[]
+    address_id=None
     if ip:
         assert is_ipv4(ip)
         address=get_object_or_404(IPv4Address,vrf=vrf,ip=ip)
+        address_id=address.id
         initial={
             "fqdn"        : address.fqdn,
             "ip"          : address.ip,
@@ -146,7 +236,7 @@ def assign_address(request,vrf_id,ip=None,new_ip=None):
         initial={}
         p=""
     if request.POST:
-        form=AssignAddressForm(request.POST)
+        form=AssignAddressForm(request.POST,vrf=vrf,address_id=address_id)
         if form.is_valid():
             assert is_ipv4(form.cleaned_data["ip"])
             assert is_fqdn(form.cleaned_data["fqdn"])
