@@ -8,7 +8,7 @@
 """
 """
 from __future__ import with_statement
-import os,logging,pty,signal,time,re,sys,signal,Queue,cPickle,tempfile
+import os,logging,pty,signal,time,re,sys,signal,Queue,cPickle,tempfile,urllib,random
 from errno import ECONNREFUSED
 from noc.sa.profiles import profile_registry
 from noc.sa.script import script_registry,ScriptSocket
@@ -18,7 +18,7 @@ from noc.sa.protocols.pm_pb2 import *
 from noc.lib.fileutils import safe_rewrite
 from noc.lib.daemon import Daemon
 from noc.lib.fsm import FSM,check_state
-from noc.lib.nbsocket import ConnectedTCPSocket,ConnectedTCPSSLSocket,SocketFactory,PTYSocket,HAS_SSL,ListenUDPSocket
+from noc.lib.nbsocket import ConnectedTCPSocket,ConnectedTCPSSLSocket,SocketFactory,PTYSocket,HAS_SSL,ListenUDPSocket,AcceptedTCPSocket,ListenTCPSocket
 from noc.lib.debug import DEBUG_CTX_CRASH_PREFIX
 from noc.lib.pmhash import pmhash
 from threading import Lock
@@ -78,7 +78,7 @@ class Service(SAEService):
             done(controller,error=e)
             return
         # Check [activator]/max_pull_config limit
-        if self.activator.factory.count_subclass_sockets(ScriptSocket)>=self.activator.config.getint("activator","max_pull_config"):
+        if not self.activator.can_run_script():
             e=Error()
             e.code=ERR_OVERLOAD
             e.text="script concurrent session limit reached"
@@ -142,6 +142,180 @@ class ActivatorSSLSocket(RPCSocket,ConnectedTCPSSLSocket):
     def on_conn_refused(self):
         self.activator_event("refused")
 ##
+## Servers
+##
+class HTTPServerSocket(AcceptedTCPSocket):
+    def __init__(self,factory,socket,server_hub):
+        super(HTTPServerSocket,self).__init__(factory,socket)
+        self.server_hub=server_hub
+        self.context=None
+        self.request=""
+        self.path=None
+        self.host=""
+        self.data_size=0
+    ##
+    ## <!> STUB
+    ##
+    @classmethod
+    def check_access(self,address):
+        return True
+    ##
+    def feed(self,data):
+        self.data_size-=len(data)
+        self.context.feed(data)
+    ##
+    ##
+    ##
+    def on_read(self,data):
+        logging.debug("HTTP Read: %s"%data)
+        if self.context is None:
+            # Parse request
+            self.request+=data
+            for s in ["\n\n","\r\n\r\n"]:
+                if s in self.request:
+                    req,data=self.request.split(s,1)
+                    l0,lo=req.split("\n",1)
+                    logging.debug(l0)
+                    r=l0.split(" ")
+                    if len(r)!=3 or r[0] not in ["PUT"]:
+                        logging.error("Invalid HTTP request")
+                        self.close()
+                        return
+                    address,port=self.socket.getpeername()
+                    self.path=urllib.unquote(r[1])
+                    try:
+                        self.context=self.server_hub.get_context("http",address,self.path)
+                    except KeyError:
+                        logging.error("Permission denied: %s %s"%(address,self.path))
+                        self.close()
+                        return
+                    for l in lo.split("\n"):
+                        l=l.strip()
+                        k,v=[x.strip() for x in l.split(":",1)]
+                        k=k.lower()
+                        if k=="content-length":
+                            self.data_size=int(v)
+                        elif k=="host":
+                            self.host=v
+                    break
+        else:
+            # Populate data
+            self.feed(data)
+            if self.data_size<=0:
+                # Return response on all data received
+                self.write("HTTP/1.1 201 Created\r\nContent-Length: 0\r\n\r\n")
+                self.close()
+
+class HTTPServer(ListenTCPSocket):
+    def __init__(self,factory,address,server_hub):
+        super(HTTPServer,self).__init__(factory,address,80,HTTPServerSocket,server_hub=server_hub)
+##
+##
+##
+class ServerContextManager(object):
+    CONTEXT_NAME=None
+    def __init__(self,server_hub):
+        self.server_hub=server_hub
+        self.address=None
+        self.path=None
+        self.data=[]
+    ##
+    ## Set request filter
+    ##
+    def prepare(self,address,path=None):
+        self.address=address
+        if path:
+            self.path=None
+        else:
+            self.path="/%s-%s-%d"%(str(id(self)),address,random.randint(0,0x7FFFFFFF))
+        return self.path
+    ##
+    ##
+    ##
+    def get_url(self,address,path=None):
+        path=self.prepare(address,path)
+        return "%s://%s%s"%(self.CONTEXT_NAME,self.server_hub.http_server_address,urllib.quote(path))
+    ##
+    ## Pupulate data
+    ##
+    def feed(self,data):
+        self.data+=[data]
+    ##
+    ## Return collected data
+    ##
+    def get_data(self):
+        return "".join(self.data)
+    ##
+    ## Register server context
+    ##
+    def __enter__(self):
+        self.server_hub.register_context(self.CONTEXT_NAME,self)
+        return self
+    ##
+    ## Unregister server context
+    ##
+    def __exit__(self,exc_type, exc_val, exc_tb):
+        self.server_hub.unregister_context(self.CONTEXT_NAME,self)
+##
+##
+##
+class HTTPServerContextManager(ServerContextManager):
+    CONTEXT_NAME="http"
+##
+## Server hub
+##
+class ServersHub(object):
+    def __init__(self,activator):
+        self.activator=activator
+        self.contexts={
+            "http" : set(),
+        }
+        self.http_server_address=self.activator.config.get("servers","listen_http")
+        self.http_server=HTTPServer(self.activator.factory,self.http_server_address,self) if self.http_server_address else None
+    ##
+    ## Register new context
+    ##
+    def register_context(self,context_name,context):
+        logging.debug("Register %s server context: %s"%(context_name,context))
+        self.contexts[context_name].add(context)
+    ##
+    ## Unregister context
+    ##
+    def unregister_context(self,context_name,context):
+        logging.debug("Unregister %s server context: %s"%(context_name,context))
+        self.contexts[context_name].remove(context)
+    ##
+    ##
+    ##
+    def get_context(self,context_name,address,path):
+        for c in self.contexts[context_name]:
+            if c.address==address and c.path==path:
+                return c
+        raise KeyError
+    
+    ##
+    ## Check IP address has access to context
+    ##
+    def check_address_access(self,context_name,address):
+        return len([c for c in self.contexts[context_name] if c.address==address])>0
+    ##
+    ## Check IP address has access to path
+    ##
+    def check_path_access(self,context_name,address,path):
+        return len([c for c in self.contexts[context_name] if c.address==address and c.path==path])>0
+    ##
+    ## Close all servers
+    ##
+    def close(self):
+        if self.http_server:
+            self.http_server.close()
+    ##
+    ## Return HTTP Server Context Manager
+    ##
+    def http(self):
+        return HTTPServerContextManager(self)
+
+##
 ## External fping process.
 ## Runs fping, supplies a list of checked hosts
 ## and reads a list uf unreachable hosts
@@ -256,12 +430,15 @@ class Activator(Daemon,FSM):
         FSM.__init__(self)
         self.next_filter_update=None
         self.next_crashinfo_check=None
+        self.next_heartbeat=None
         self.script_threads={}
+        self.max_script_threads=self.config.getint("activator","max_pull_config")
         self.script_lock=Lock()
         self.script_call_queue=Queue.Queue()
         self.pm_data_queue=[]
         self.pm_result_queue=[]
         self.pm_data_secret=self.config.get("activator","pm_data_secret")
+        self.servers=ServersHub(self)
     ##
     ## IDLE state 
     ##
@@ -393,6 +570,13 @@ class Activator(Daemon,FSM):
                 pdc.close()
             self.pm_data_collectors=[]
     ##
+    ## Checks wrether max_pull_config limit exceeded
+    ##
+    def can_run_script(self):
+        with self.script_lock:
+            return len(self.script_threads)<self.max_script_threads
+    
+    ##
     ## Script support
     ##
     def run_script(self,name,access_profile,callback,**kwargs):
@@ -433,11 +617,12 @@ class Activator(Daemon,FSM):
         self.factory.run(run_forever=True)
     ##
     def tick(self):
+        t=time.time()
         # Request filter updates
-        if self.get_state()=="ESTABLISHED" and self.next_filter_update and time.time()>self.next_filter_update:
+        if self.get_state()=="ESTABLISHED" and self.next_filter_update and t>self.next_filter_update:
             self.get_event_filter()
         # Check for pending crashinfos
-        if self.stand_alone_mode and self.next_crashinfo_check and time.time()>self.next_crashinfo_check and self.get_state()=="ESTABLISHED":
+        if self.stand_alone_mode  and self.get_state()=="ESTABLISHED" and self.next_crashinfo_check and t>self.next_crashinfo_check:
             self.check_crashinfo()
         # Perform delayed calls
         while not self.script_call_queue.empty():
@@ -453,6 +638,10 @@ class Activator(Daemon,FSM):
         # Cancel stale scripts
         if self.get_state()=="ESTABLISHED":
             self.cancel_stale_scripts()
+        # Heartbeat when necessary
+        if self.heartbeat_enable and (self.next_heartbeat is None or self.next_heartbeat<=t):
+            self.heartbeat()
+            self.next_heartbeat=t+3
         # Perform default daemon/fsm machinery
         super(Activator,self).tick()
                 
@@ -571,7 +760,9 @@ class Activator(Daemon,FSM):
                 logging.error("get_event_filter error: %s"%error.text)
                 return
             self.event_sources=set(response.sources)
+            logging.debug("Setting event source filter to: %s"%str(self.event_sources))
             self.next_filter_update=time.time()+response.expire
+        logging.info("Requesting event source filter")
         r=EventFilterRequest()
         self.sae_stream.proxy.event_filter(r,event_filter_callback)
     ##
@@ -653,8 +844,9 @@ class Activator(Daemon,FSM):
     ##
     ## Cancel stale scripts
     ##
+    @check_state("ESTABLISHED")
     def cancel_stale_scripts(self):
-        to_cancel=[st for st in self.script_threads if st.is_stale()]
+        to_cancel=[st for st in self.script_threads.keys() if st.is_stale()]
         for script in to_cancel:
             logging.info("Canceling stale script %s(%s)"%(script.name,script.access_profile.address))
             st.cancel_script()
