@@ -16,7 +16,7 @@ class Profile(noc.sa.profiles.Profile):
     supported_schemes=[SSH]
     pattern_username="^([Uu]sername|[Ll]ogin):"
     pattern_prompt="bp>"
-    pattern_more=r"^(/var/tmp/shell\.out\.|:)"
+    pattern_more=r"^(/var/tmp/shell\.out\.\S+|:)"
     pattern_more_end=r"^\(END\)"
-    command_more=" "
+    command_more="w1000000\n"
     command_more_end="q"
